@@ -13,8 +13,8 @@
 # limitations under the License.
 
 
+import inspect
 import os
-import os.path
 import time
 import types
 from test_case import MetaTestCase
@@ -56,7 +56,12 @@ def discover(what):
                 import_error = e
                 _log.info('discover_inner: Failed to import %s: %s' % (locator, e))
                 if os.path.isfile(locator) or os.path.isfile(locator+'.py'):
-                    new_loc = os.path.relpath(os.path.normpath(locator))
+                    here = os.path.abspath(os.path.curdir) + os.path.sep
+                    new_loc = os.path.abspath(locator)
+                    if not new_loc.startswith(here):
+                        raise DiscoveryError('Can only load modules by path within the current directory')
+
+                    new_loc = new_loc[len(here):]
                     new_loc = new_loc.rsplit('.py',1)[0] #allows for .pyc and .pyo as well
                     new_loc = new_loc.replace(os.sep,'.')
                     try:
@@ -117,7 +122,7 @@ def discover(what):
                 module_suites = getattr(test_module, '_suites', [])
                 for member_name in dir(test_module):
                     obj = getattr(test_module, member_name)
-                    if isinstance(obj, types.TypeType):
+                    if isinstance(obj, types.TypeType) and inspect.getmodule(obj) == test_module:
                         for test_case_class in discover_inner(obj, suites + module_suites):
                             yield test_case_class
 
